// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the ATLAS_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// ATLAS_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef ATLAS_EXPORTS
#define ATLAS_API __declspec(dllexport)
#else
#define ATLAS_API __declspec(dllimport)
#endif

/*! Use floats or doubles in Atlas? */
typedef float AT_scalar;
//typedef double AT_scalar;

/*! Callback function type for distance calculation. */
typedef AT_scalar (* AT_CalculateDistanceCallbackType) (int vID0, int vID1, void *userDef);
typedef AT_scalar (* AT_AstarHeuristicCallbackType) (int vID0, int vID1, void *userDef);

/*!
  \defgroup Vertex Vertex functions
*/
/*!
  \defgroup Edge Edge functions
*/
/*!
  \defgroup Query Query functions
*/
/*!
  \defgroup Retrieval Retrieval functions
*/
/*!
  \defgroup Misc Misc functions
*/

//! Create and maintain a graph and find shortest paths.
/*! */

/*! The function was succesful. */
#define AT_SUCCESS                  0
/*! You provided an illegal value for one or more of the parameters. */
#define AT_ILLEGAL_VALUE           -1
/*! You try to add an edge that already exists. */
#define AT_EDGE_ALREADY_EXISTS     -2
/*! The query did not yield a path. */
#define AT_NO_PATH_FOUND           -3
/*! The vertex ID does not exist. */
#define AT_NO_SUCH_VERTEX          -4
/*! The edge does not exist. */
#define AT_NO_SUCH_EDGE            -5
/*! The operation you try to perform is only valid on undirected graphs. */
#define AT_GRAPH_DIRECTED          -6
/*! You changed the graph and need to recalculate the shortest paths before querying. */
#define AT_NEED_RECALCULATION      -7

//! These are all functions exported by Atlas.
class ATLAS_API CAtlas {
	//added by Arno
private:
	void* main;
	//end added by Arno
public:
  /*! This function creates an instance of Atlas.
	  \param dimension The dimension of the vertices.
    \param directed Set to TRUE if the graph is directed.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Misc
  */
  CAtlas (int dimension, bool directed);

  /*! The destructor for the class.
    \ingroup Misc
  */
  virtual ~CAtlas ();

  /*! Retrieve the dimension of the vertices.
    \param dimension The dimension.
    \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Misc
  */
  int GetDimension (int *dimension);

  // vertex functions
  /*! This function adds a vertex to the graph.
	  \param conf A list of length dimension containing the values of the vertex.
    \param vID A return value contaning the ID of the vertex.
    \param userDef An optional pointer to store user defined data with the vertex.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int AddVertex (AT_scalar *conf, int *vID, void *userDef=NULL);

  /*! This function removes a vertex from the graph.
	  \param vID The ID of the vertex to remove.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int RemoveVertex (int vID);

  /*! Use this function to retrieve information from the vertex.
	  \param vID The ID of the vertex.
    \param conf Return value: the list of vertex values, must be of the proper size, can be NULL if you are only interested in userDef.
    \param userDef Return value: optional, the user defined pointer associated with the vertex.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetVertexConfiguration (int vID, AT_scalar* conf, void **userDef=NULL);

  /*! This function retrieves the total number of vertices in the graph.
	  \param nr The number of vertices.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetNrVertices (int *nr);

  /*! This function retrieves the vertices in the graph.
	  \param vIDs The list of vertices. This list needs to have the proper size.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetVertices (int *vIDs);

  /*! This function enables the user to set the user defined pointer at a later moment.
	  \param vID The ID of the vertex.
    \param userDef The pointer to the user defined data.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int SetUserDef (int vID, void *userDef);

  // edge functions
  /*! Adds a connection between two vertices.
	  \param vID0 The ID of the first vertex.
    \param vID1 The ID of the second vertex.
    \param value The value of the edge, used by finding shortest paths.
    \param userDef An optional pointer to store user defined data with the edge.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Edge
  */
  int AddEdge (int vID0, int vID1, AT_scalar value, void *userDef=NULL);

  /*! This function removes a connection from the graph.
	  \param vID0 The ID of the first vertex.
    \param vID1 The ID of the second vertex.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Edge
  */
  int RemoveEdge (int vID0, int vID1);

  /*! Revrieve the number of incoming edges of a vertex.
    \param vID The ID of the vertex.
    \param nr The nr of connections to the vertex.
    \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetNrPredecessors (int vID, int *nr);

  /*! Revrieve the number of outgoing edges of a vertex.
    \param vID The ID of the vertex.
    \param nr The nr of connections from the vertex.
    \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetNrSuccessors (int vID, int *nr);

  /*! Retrieve the ID of the nr'th successor.
    \param vID The ID of the vertex.
    \param nr The index of the successor.
    \param nID The ID of the successor.
    \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetSuccessor (int vID, int nr, int *nID);

  /*! Retrieve the ID of the nr'th predecessor.
    \param vID The ID of the vertex.
    \param nr The index of the predecessor
    \param nID The ID of the predecessor.
    \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetPredecessor (int vID, int nr, int *nID);

  /*! Retrieve all predecessor ID's a vertex.
    \param vID The ID of the vertex.
    \param ids The list of predecessor vertex ID's (needs to be the right size).
    \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetPredecessors (int vID, int *ids);

  /*! Retrieve all successor ID's a vertex.
    \param vID The ID of the vertex.
    \param ids The list of successor vertex ID's (needs to be the right size).
    \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetSuccessors (int vID, int *ids);

  /*! This function sets the value of an edge.
	  \param vID0 The ID of the first vertex.
    \param vID1 The ID of the second vertex.
    \param value The new value for the edge.
    \param userDef An optional pointer to store user defined data with the vertex.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Edge
  */
  int SetEdgeConfiguration (int vID0, int vID1, AT_scalar value, void *userDef=NULL);

  /*! This function retrieves the value of an edge.
	  \param vID0 The ID of the first vertex.
    \param vID1 The ID of the second vertex.
    \param value The value of the edge.
    \param userDef Return value: optional, the user defined pointer associated with the vertex.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Edge
  */
  int GetEdgeConfiguration (int vID0, int vID1, AT_scalar *value, void **userDef=NULL);

  /*! Temporarily invalidate an edge.
	  \param vID0 The ID of the first vertex.
    \param vID1 The ID of the second vertex.
    \param invalidate Set to true or false.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Edge
  */
  int SetEdgeInvalidation (int vID0, int vID1, bool invalidate);

  // query functions
  /*! This function calculates a single source shortest path.
	  \param startID The ID of the vertex from which to calculate the shortest paths.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Query
  */
  int FindShortestPaths (int startID);

  /*! This function calculates an all pairs shortest path.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Query
  */
  int FindAllPairsShortestPaths ();

  /*! Perform an A* search between a start and goal vertex.
  */
  int AStar (int startID, int goalID, AT_AstarHeuristicCallbackType ASHCB, int *size, void *userDef=NULL);

  int GetAStarQueryVertex (int nr, int *vID);

  /*! Query for a shortest path. First use FindShortestPaths!
	  \param goalID The ID of the goal vertex.
    \param size Return value for the number of vertices in the query path.
    \param length Return value that holds the length of the path.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Query
  */
  int Query (int goalID, int *size, AT_scalar *length=NULL);

  /*! Query for a shortest path, created by all pairs shortest path. First use FindAllPairsShortestPahts!
	  \param startID The ID of the start vertex.
    \param goalID The ID of the goal vertex.
    \param size Return value for the number of vertices in the query path.
    \param length Return value that holds the length of the path.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Query
  */
  int Query (int startID, int goalID, int *size, AT_scalar *length=NULL);

  /*! Retrieve a vertex in the query.
	  \param nr The number of the query vertex. The total numer is returned by the Query function.
    \param vID Return value for the ID of the vertex in the query.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Query
  */
  int GetQueryVertex (int nr, int* vID);

  /*! Check whether two vertices are connected, either direct or via other vertices.
	  \param vID0 The ID of the first vertex.
    \param vID1 The ID of the second vertex.
    \param connected Return value whether the two vertices are connected.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int VerticesConnected (int vID0, int vID1, bool *connected);

  //misc
  /*! Delete graph.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Misc
  */
  int ResetGraph ();

  /*! Loads a graph from disc.
    \param fileName Name of the file to load.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Misc
  */
  int LoadGraph (char *fileName);

  /*! Saves a graph to disc.
    \param fileName Name of the file to save.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Misc
  */
  int SaveGraph (char *fileName);

  /*! Checks whether an edge that creates a cycle is "useful".
    \param startID The ID of the start vertex.
    \param goalID The ID of the goal vertex.
    \param K The K value to uses when determining the usefulness.
    \param calcDistCB A user provided callback routine to calculate the discance between two vertices.
    \param result True when the edge is useful.
    \param userDef A user defined pointer that is send to the callback function.
    \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Misc
  */
  int CheckUsefulCycle (int startID, int goalID, AT_scalar K, AT_CalculateDistanceCallbackType calcDistCB, bool *result, void *userDef=NULL);
};

/*! \file atlas.h
*/